@@ -35,8 +35,10 @@ components allow you to collect input from the user.
 
 # pylint:disable=unused-argument
 
+from . import logger
 import functools
 import logging
+logger = logging.getLogger("Edifice")
 import os
 import typing as tp
 
@@ -292,7 +294,7 @@ class QtWidgetComponent(WidgetComponent):
                 elif style["align"] == "bottom":
                     set_align = QtCore.Qt.AlignBottom
                 else:
-                    logging.warning("Unknown alignment: %s", style["align"])
+                    logger.warning("Unknown alignment: %s", style["align"])
                 style.pop("align")
 
             if set_margin:
@@ -314,7 +316,7 @@ class QtWidgetComponent(WidgetComponent):
                 elif style["align"] == "bottom":
                     set_align = "AlignBottom"
                 else:
-                    logging.warning("Unknown alignment: %s", style["align"])
+                    logger.warning("Unknown alignment: %s", style["align"])
                 style.pop("align")
                 style["qproperty-alignment"] = set_align
 
@@ -332,18 +334,18 @@ class QtWidgetComponent(WidgetComponent):
                 style["min-width"] = style["width"]
             if "max-width" not in style:
                 style["max-width"] = style["width"]
-        else:
-            if "min-width" not in style:
-                style["min-width"] = self._get_width(children)
+        # else:
+        #     if "min-width" not in style:
+        #         style["min-width"] = self._get_width(children)
 
         if "height" in style:
             if "min-height" not in style:
                 style["min-height"] = style["height"]
             if "max-height" not in style:
                 style["max-height"] = style["height"]
-        else:
-            if "min-height" not in style:
-                style["min-height"] = self._get_height(children)
+        # else:
+        #     if "min-height" not in style:
+        #         style["min-height"] = self._get_height(children)
 
         set_move = False
         move_coords = [0, 0]
@@ -383,7 +385,7 @@ class QtWidgetComponent(WidgetComponent):
                 style = newprops[prop] or {}
                 if isinstance(style, list):
                     # style is nonempty since otherwise the or statement would make it a dict
-                    first_style = style[0]
+                    first_style = style[0].copy()
                     for next_style in style[1:]:
                         first_style.update(next_style)
                     style = first_style
@@ -442,6 +444,10 @@ class Window(RootComponent):
         self._menu_bar = None
         self.underlying = None
 
+    def will_unmount(self):
+        if self._previous_rendering:
+            self._previous_rendering.underlying.close()
+
     def _set_on_close(self, underlying, on_close):
         self._on_close = on_close
         if on_close:
@@ -519,7 +525,12 @@ class GroupBox(QtWidgetComponent):
 class Icon(QtWidgetComponent):
     """Display an Icon
 
-    Icons are fairly central to modern-looking UI design.
+    .. figure:: ../widget_images/icons.png
+       :width: 300
+
+       Two icons. Note that you can set the color and rotation.
+
+    Icons are central to modern-looking UI design.
     Edifice comes with the Font Awesome (https://fontawesome.com) regular and solid
     icon sets, to save you time from looking up your own icon set.
     You can specify an icon simplify using its name (and optionally the sub_collection).
@@ -575,6 +586,11 @@ class Icon(QtWidgetComponent):
 class Button(QtWidgetComponent):
     """Basic button widget.
 
+    .. figure:: ../widget_images/textinput_button.png
+       :width: 300
+
+       Button on the right
+
     Set the on_click prop (inherited from QtWidgetComponent) to define the behavior on click.
 
     Args:
@@ -608,6 +624,11 @@ class Button(QtWidgetComponent):
 class IconButton(Button):
     """Display an Icon Button.
 
+    .. figure:: ../widget_images/icons.png
+       :width: 300
+
+       Icon button on the very right.
+
     Icons are fairly central to modern-looking UI design;
     this component allows you to put an icon in a button.
     Edifice comes with the Font Awesome (https://fontawesome.com) regular and solid
@@ -659,6 +680,11 @@ class IconButton(Button):
 class Label(QtWidgetComponent):
     """Basic widget for displaying text.
 
+    .. figure:: ../widget_images/label.png
+       :width: 500
+
+       Three different label objects. You can embed HTML in labels to get rich text formatting.
+
     Args:
         text: the text to display. Can be any Python type; the text prop is converted
             to a string using str before being displayed
@@ -726,6 +752,11 @@ class Completer(object):
 class TextInput(QtWidgetComponent):
     """Basic widget for a one line text input
 
+    .. figure:: ../widget_images/textinput_button.png
+       :width: 300
+
+       TextInput on the left. Note that you can set an optional Completer, giving the dropdown for completion.
+
     Args:
         text: Initial text of the text input
         on_change: callback for the value of the text input changes. The callback is passed the changed
@@ -734,9 +765,11 @@ class TextInput(QtWidgetComponent):
 
     @register_props
     def __init__(self, text: tp.Any = "", on_change: tp.Callable[[tp.Text], None] = (lambda text: None),
+                 on_edit_finish: tp.Callable[[], None] = (lambda: None),
                  completer: tp.Optional[Completer] = None, **kwargs):
         super().__init__(**kwargs)
-        self._connected = False
+        self._on_change_connected = False
+        self._editing_finished_connected = False
         self.underlying = None
 
     def _initialize(self):
@@ -748,10 +781,18 @@ class TextInput(QtWidgetComponent):
     def set_on_change(self, on_change):
         def on_change_fun(text):
             return on_change(text)
-        if self._connected:
+        if self._on_change_connected:
             self.underlying.textChanged[str].disconnect()
         self.underlying.textChanged[str].connect(on_change_fun)
-        self._connected = True
+        self._on_change_connected = True
+
+    def set_on_edit_finish(self, on_edit_finish):
+        def on_edit_finish_fun():
+            return on_edit_finish()
+        if self._editing_finished_connected:
+            self.underlying.editingFinished.disconnect()
+        self.underlying.editingFinished.connect(on_edit_finish_fun)
+        self._editing_finished_connected = True
 
     def set_completer(self, completer):
         if completer:
@@ -770,6 +811,8 @@ class TextInput(QtWidgetComponent):
         for prop in newprops:
             if prop == "on_change":
                 commands.append((self.set_on_change, newprops[prop]))
+            elif prop == "on_edit_finish":
+                commands.append((self.set_on_edit_finish, newprops[prop]))
             elif prop == "completer":
                 commands.append((self.set_completer, newprops[prop]))
         return commands
@@ -778,6 +821,11 @@ class TextInput(QtWidgetComponent):
 class Dropdown(QtWidgetComponent):
     """Basic widget for a dropdown menu.
 
+    .. figure:: ../widget_images/checkbox_dropdown.png
+       :width: 300
+
+       Dropdown on the right.
+
     Args:
         text: Initial text of the text input
         on_change: callback for the value of the text input changes. The callback is passed the changed
@@ -786,10 +834,9 @@ class Dropdown(QtWidgetComponent):
 
     @register_props
     def __init__(self, text: tp.Text = "", selection: tp.Text = "",
-                 editable: bool = False,
-                 show_on_type: bool = True,
-                 completer: tp.Optional[Completer] = None,
                  options: tp.Optional[tp.Sequence[tp.Text]] = None,
+                 editable: bool = False,
+                 completer: tp.Optional[Completer] = None,
                  on_change: tp.Callable[[tp.Text], None] = (lambda text: None),
                  on_select: tp.Callable[[tp.Text], None] = (lambda text: None),
                  **kwargs):
@@ -853,6 +900,11 @@ class Dropdown(QtWidgetComponent):
 class CheckBox(QtWidgetComponent):
     """Checkbox widget.
 
+    .. figure:: ../widget_images/checkbox_dropdown.png
+       :width: 300
+
+       Checkbox on the left.
+
     A checkbox allows the user to specify some boolean state.
 
     The checked prop determines the initial check-state of the widget.
@@ -907,6 +959,11 @@ NumericType = tp.Union[float, int]
 class Slider(QtWidgetComponent):
     """Slider bar widget.
 
+    .. figure:: ../widget_images/slider.png
+       :width: 300
+
+       Horizontal and vertical sliders
+
     A Slider bar allows the user to input a continuous value.
     The bar could be displayed either horizontally or vertically.
 
@@ -1116,6 +1173,11 @@ class View(_LinearView):
 class ScrollView(_LinearView):
     """Scrollable layout widget for grouping children together.
 
+    .. figure:: ../widget_images/scroll_view.png
+       :width: 500
+
+       A ScrollView containing a Label.
+
     Unlike :doc:`View<edifice.base_components.View>`, overflows in both the x and y direction
     will cause a scrollbar to show.
 
@@ -1205,7 +1267,7 @@ class CustomWidget(QtWidgetComponent):
         return commands
 
 
-class List(BaseComponent):
+class List(RootComponent):
 
     @register_props
     def __init__(self):
